//----------------------------------------------------------------------------*/
/* File: 133M_VFO.C					Stand 25.10.05                                    */
/* Autor: Guenter Zobel DM2DSN                                                */
/* Steuerprogramm des DDS-VFO 133MHz mit ATMEGA8515                           */
/* Port A Pins vom ATMEGA8515 verbunden mit LCD D0...D7 (pins 7...14).        */ 
/* PD4  -->   LCD ('WR).                                                      */
/* PD5  -->   LCD (RS).                                                       */
/* PD6  --> LCD (E)                                                           */
/* Drehgeber Impuls A--> PD2 (Interrupt)                                      */
/*           Impuls B--> PD7                                                  */
/*                                                                            */
/* Resonator frequency = 8 MHz                                                */
/* Programmierung der Taktauswahlbits  CKSEL 3..0: 1111 (hochfrequenter Quarz)*/
/*----------------------------------------------------------------------------*/

 #include <io8515v.h> 
 #include <eeprom.h>
 #pragma interrupt_handler timer:5 radimpuls:2 ausschalten:3
 
 
 // Makros zur LCD-Anzeige
 #define LCD_E  0x40
 #define LCD_RS 0x20
 #define LCD_RW 0x10
 #define lcd_set_e()  (PORTD |= LCD_E)
 #define lcd_set_rs() (PORTD |= LCD_RS)
 #define lcd_set_rw() (PORTD |= LCD_RW)
 #define lcd_clear_e()  (PORTD &= ~LCD_E)
 #define lcd_clear_rs() (PORTD &= ~LCD_RS)
 #define lcd_clear_rw() (PORTD &= ~LCD_RW)
 
 // Makros zur Tasterabfrage
 #define SP1_set   0x01 
 #define SP1_read  0x02 
 #define SP2_set   0x04 
 #define SP2_read  0x08  
 #define Baken     0x10
 #define Modus     0x20
 #define RIT       0x40 
 #define Tx        0x80  
 
 // Makros zum bNormal-Byte   
 #define Normal_OK    0x80

   
 #define f_ZF    1070000    		// Frequenz der ZF
 
// Globale Variablen 
int iSegmente=0; //Vorzeichenbehaftete Anzahl der Segmente, die von der Inter-
                 //  ruptroutine "radimpuls" ermittelt wurden
unsigned char bTicks=0;  //zaehlt 50 ms Intervalle               
unsigned char i;		// Laufvariable  
char buf[16];
unsigned bModus;           
unsigned char bAkt_Einstellung; //Wird in den EEPROM gerettet
//___7_____________________________________3__________2___________1_______0____
//|       |        |        |1: Baken||1: RIT    | Modusnummer 0..7            | 
//|       |        |        |0: keine||0: kein R.|0:2m-Betrieb,1:GHz-Betrieb   |
//|       |        |        |   B.   ||          |2: 10GHz-Band                |      
// -----------------------------------------------------------------------------  
//                     Rx        Tx 
unsigned char bParam = 0; // Parameter fuer Drehgeberfunktion
//___7_____________________________________________________________________0_____
//|2m_f/ |  Senden/  | 0:1.Zeile |      ||Reserve | Reserve | Reserve |  Reserve | 
//|70_f\ |nicht Send.| 1:2.Zeile |      ||        |         |         |          |
// ------------------------------------------------------------------------------  
 unsigned char bTaster_v= 0 ;   // Tasterstell. vom letzten Durchlauf
	                              // zum Vergleichen gespeichert, 
 unsigned char bTaster_k= 0 ;   // entprellte  Tasterstellung
                                // zum Vergleichen gespeichert    
 unsigned char bTaster= 0 ;     //Inhalt sind d. aktuellen gueltigen Zustaende
                          
//___7_____________________________________________________________________0_____
//| TX/Rx\ |  RIT  |  Modus  | Baken || SP2 read | SP2 set | SP1 read | SP1 set.| 
// ------------------------------------------------------------------------------ 
union Zuordnung1
 	{unsigned long lZahl;
 	 unsigned char Byte[4];
 	}                         
  	 uDDS_Frequenz,		// Frequenz die seriell zum DDS-IC geschickt wird 
 	   uRIT_Rel;	// Sendefrequenz bei RIT-Betrieb (kein EEPROM)
 

 union Zuordnung1 *pRIT_Rel; //Pointer der Art "union Zuordnung1" aber noch
                             //  nicht ausgerichtet auf eine solche union 
 union Zuordnung2
	{unsigned long lZahl[3];
	 unsigned char Byte[12];
	}uTxRx;
 
 unsigned long *p2m_Rx;	  //Zeiger, wird noch auf uTxRx.lZahl[0] gerichtet
 unsigned long *pBaken2m;	//            "         uTxRx.lZahl[1]    "
 unsigned long *pBakenGHz;	//            "         uTxRx.lZahl[2]    "  
 unsigned char Frequenz1[5]; //Array das mit FTW + Adresse von FTW gefuellt wird
 unsigned char Steuerregister [5];//Zum Fuellen der Steuerreg. des AD9951
 unsigned char bGelesen;
 char bNormal_OK = 0 ;		// Zustandsbyte  bei Normalbetrieb
 //___7_________________________________________________0___
 //|Normal| SSB |  FM  |  USB  || LSB |Merk. |      |       | 
 //|  Ok  | Ok  |  Ok  |  Ok   || Ok  |0:Rx  |      |       |
 //|      |     |      |       ||     |1:Tx  |      |       |
 // -------------------------------------------------------- 

//___7_________________________________________________0___
//|Split |Merk.| Merk.| Merk. ||     |      |      |       | 
//|  Ok  |0:Rx |0:SSB |0:USB  ||     |      |      |       |
//|      |1:TX |1:FM  |1:LSB  ||                           |
// -------------------------------------------------------- 
unsigned char bOK_Byte=0;		// Betriebsarten Kenn-Nr.
unsigned char bBetriebsart_zeigen;	// 0--> kein -,  1-->Anzeigetext auf LCD bringen      
/********************************************************************************/ 	
/* Funktion zur Bildung der n-Potenz zur Basis base   (long)			              */
/*										                                                          */
/********************************************************************************/
long lPower(unsigned char base, unsigned char n)
{
 unsigned char i; 
 long p =1;
 for (i=1; i<=n; i++)
 	p=p*base;
 return p;
}

/********************************************************************************/
/* Funktion zum Umwandeln einer unsigned long-Zahl in einen nullterminierten 	  */
/* String. Fuer buf mu ein Stringarray von min. 15 Zeichen bereitstehen	      */
/* Wertebereich: 4 294 967 295,Darstellung im Moment bis 999.999,99  	          */
/* Zur besseren Lesbarkeit werden aller 3 Stellen '.'-Zeichen eingefgt	  	    */ 
/* Erfordert: unsigned long lPower(int base, int n)				                      */
/********************************************************************************/
void sprint_long (char* buf, unsigned long lZahl, unsigned char Parameter)
{

   unsigned char i, z, Merker, Index=0; 
   
   if (Parameter ==2) //10 GHz-Anzeige
        lZahl += 22400000; //Damit aus 144 000 00 -> 368 000 00 wird
        
   for (i=9; i>=1; i--) 	//Umwandeln des Betrages in String
   {
   	   Merker = lZahl/(lPower(10, i));
   	   buf[Index] = Merker + 48 ;
	     Index++;
	     lZahl-= Merker * (lPower(10, i));
   }

   buf[Index]=lZahl+48;		// Einerstelle
   Index++;
   buf[Index]=0;		// abschlieende Null
/* Unterdrueckung der Vornullen */ 
   
   for (i=0; i<Index; i++)
   {

   	while (buf[i]=='0')
   	{
   		for (z=i; z<Index; z++) 
   		  buf[z]=buf[z+1];  
      Index--;
   	}
   	if(buf[i]!='0')	// wenn 1...9
   	{
   		if(buf[0]==0)
   		{
   			buf[0]='0';	// letzte Null wieder einfuegen
   			buf[1]=0;
   		}
   		z= strlen(buf);

   	       if (z > 5 )
   			{for (i=z; i>(z-6); i--)	//nach rechts schieben
   			 	buf[i+1]=buf[i];
   			 buf[z-5]='.';			// "." einfgen
   			 for (i=z+1; i>(z-2); i--)	//wieder schieben
   			 	buf[i+1]=buf[i];   			 
   			 buf[z-1]=',';			//"," einfgen
   			}
   		else if (z >= 2)
   			{buf[z+1]=buf[z];
   			 buf[z]=buf[z-1];
   			 buf[z-1]=buf[z-2];
   			 buf[z-2]=',';
   			}   		
   		return;
   	}
   }
   return;
   	
}

/********************************************************************************/
/* Funktionen zum Gebrauch des LCD				 	                                    */
/********************************************************************************/

  // Kurze Verzoegerung fuer LCD
 void delay(int ticks)
  {
  while(ticks--);
  }
 
  // Strobe-Impuls fuer LCD
 void lcd_pulse(void)//Anlegen eines kurzen Enable-Impulses, am Ende (H/L-Flanke)
  {		     //werden angelegte Daten wirksam!
  lcd_set_e();	     // bei 8MHz: 8
  delay(8);
  lcd_clear_e();
  delay(8);
  }    

 // Ein Byte zur LCD senden
 void lcd_send(unsigned char data)
  {
  delay(2000);			// bei 8MHz 2000
  PORTA = data;  
  lcd_pulse();	//Zur Uebernahme der Daten wird eine kurze Enable-Nadel angelegt
  }

 //  "Clear Display": Loescht
 //                  das Display und setzt den Cursor auf die Homeposition     
 void clrscr(void)
  {      
  lcd_clear_rs();	//RS-Bit zuruecksetzen
  lcd_clear_rw();	//RW-Bit zuruecksetzen
  lcd_send(0x01);	// im Datenbyte ist nur das kleinste Bit gesetzt
  delay(2000);	// um den clear-Befehl zu realisieren, 8MHz 2000
  }

 // Initialisierung des Display
 void init_disp(void)
  {
  unsigned int j;	

  for (j=0; j < 0x1FF0; j++) /* wait loop > 15ms, vorher FF0  */
        ;
  lcd_clear_rs();	//RS-Bit zuruecksetzen
  lcd_clear_rw();	//RW-Bit zuruecksetzen
  lcd_send(0x30);	//Function->Set: Datenbusbreite 8 Bit,Zweizeil. Display(!),
  			//		5*7 Punkte pro Zeichen  
  for (j=0; j < 0xBE0; j++) /* wait loop > 4ms(5F0) */
        ;						
  lcd_send(0x30);	// Nochmal (?)
  for (j=0; j < 0x1E0; j++) 		//vorher F0
        ;
  lcd_send(0x30);	// und nochmal!
  //for (j=0; j < 0x1F0; j++)
    //  ;
  lcd_send(0x38); // 8Bit Interface, 2 Zeilen, 5*7 Zeichenfont          
             
  lcd_send(0x0C);	//Display an
  //lcd_send(0x01);	//Display loeschen

  lcd_send(0x06);	//Adresspointer inkrementieren,Displayinh. nicht schieben
  
  }    

 void gotoz(unsigned char z)
  {
  lcd_clear_rs();
  lcd_clear_rw();
  lcd_send(z | 0x80);
  }
 #define gotoxy(x,y) gotoz((x)|((y)<<6))
 
 // Ausgabe eines einzelnen Zeichens
 void putchar2(char c)
  {
  lcd_clear_rw();	
  lcd_set_rs();
  lcd_send(c);
  } 
 // Ausgabe eines Strings zum Display
 void outtext(char* text)
 {
  unsigned char i;
  for(i = 0; text[i] && i < 16; i++)
   putchar2(text[i]);
 }

/*******************************************************************************/
/* Funktion zur Bearbeitung der Impulse vom Drehgeber             	           */
/* Parameter: Pointer, der auf die Frequenz zeigt, die gerade bearbeitet wird  */
/*global: unsigned char bParam:                                                */
/*					  Bit 6  = 0-> nicht zum AD9951 funken                             */
/*					    	     1-> zum AD9951 funken                                   */
/*					  Bit 5  = 0-> in 1. Displayzeile eintragen                        */
/*						         1-> in 2. Displayzeile eintragen                        */
/*         unsigned char bAkt_Einstellung:	                                   */
/*            Bit 3  = 0-> kein RIT                                            */
/*						         1-> RIT                                                 */	
/*global: unsigned char bModus:                                                */	 
/*                     0-> 2m-Modus, volle Darstellung der Frequenz in Zeile 1 */
/*                     1-> Mikrowellenmodus, 2 Stellen weniger                 */     	
/*******************************************************************************/  
    
void Drehgeberauswertung(unsigned long *pFrequenz)
  {
   if(iSegmente<20 && iSegmente>-20)	      // Wenn langsam gedreht wird..
   					                                //vorher:10, ganz vorher 5,
  	*pFrequenz +=(unsigned long)(5*iSegmente);
   else if (iSegmente<40 && iSegmente>-40)	// Wenn schneller gedreht wird..
   						                              // vorher 20
	*pFrequenz +=(unsigned long)(50*iSegmente);
       else  *pFrequenz+=(unsigned long)(500*iSegmente);//Wenn schnell gedreht wird 
        
   // Neuen Wert auf Bereichsverletzung untersuchen:  

   if (*pFrequenz < 14400000)
   	    *pFrequenz =14400000;
   if (*pFrequenz > 14800000)
   	*pFrequenz =14800000; 
   sprint_long(buf, *pFrequenz, bModus);   	// uebergebene Frequenz in ASCII-String 	 
         				                            // umwandeln und in buf schreiben
   if (bParam & 0x20)		// wenn in 2. Zeile eintragen...   
   {
   	//if (bAkt_Einstellung & 0x08) // // Wenn RIT-Betrieb
   	if (bModus < 2) //wenn 2m- oder GHz-Mode..
   		  gotoxy (4,1);		   	
   	else gotoxy(8,1);	// im 10 GHz Mode
   	for (i=0; i<7; i++)  	// 4 Stellen nach links verschieben
   	  	buf[i] = buf[i+4];   	 
   }	
   else     // //wenn in 1.Zeile geschrieben wird
   {
   	if (bModus ==0)   // 2m Modus 
        gotoxy(0,0); 
    if (bModus == 1)  // Mikrowellenmodus
    {
    	for (i=0; i<9; i++)  	// 2 Stellen nach links verschieben
   	  	buf[i] = buf[i+2];
   	  gotoxy (2,0);	
    }
    if (bModus == 2)  // 10 GHz-Band-Modus
      gotoxy (4,0);	
               
   }
   outtext(buf);
   iSegmente=0;

   if (!(bParam & 0x40))	// wenn nichts zum AD9951 gesendet wird..
   	    return;				//                 		.. raus

   uDDS_Frequenz.lZahl = (unsigned long) (*pFrequenz - f_ZF)/2;
   uDDS_Frequenz.lZahl = (unsigned long) (uDDS_Frequenz.lZahl *256/5*4/5*4/5*8/5*4/5*4/3*4/3*4/3*2/13*16);
  
// Frequenz an AD9951 bermitteln: (in Frequenz[0] steht bereits die Adresse 4)
// Da MSB first ausgewaehlt ist, muss zuerst das hoehstwertigste Byte kommen:
   Frequenz1[1] = uDDS_Frequenz.Byte[3];    
   Frequenz1[2] = uDDS_Frequenz.Byte[2];    
   Frequenz1[3] = uDDS_Frequenz.Byte[1];
   Frequenz1[4] = uDDS_Frequenz.Byte[0];        
  
   PORTB = 0x00;			// bertragung beginnen, CS = 0....
                    // kein RESET mehr  
   for (i=0; i<5; i++)
        {
  	      SPDR = Frequenz1 [i];
  	      while (!(SPSR & 0x80))		// Warten auf "hoch"
  	     		; 
        }
   PORTB = 0x0C;   // CS=1, I/O UPDATE=1, RESET=0 Ende der SPI Uebertragung    
  }  
/********************************************************************************/
/* Interruptroutine bearbeitet den Interrupt vom Drehgeber, Vektor Nr.2		      */
/********************************************************************************/      
void radimpuls(void)
  {
   if(PIND & 0x80)	//Wenn vorwaerts gedreht wird...
  	iSegmente--;	//..Zahl um eins senken und raus
   else iSegmente++;		//.. ansonsten eins hinzufuegen und raus	
  } 
/********************************************************************************/
/* Interruptroutine bearbeitet den Interrupt vom Komparator            	        */
/* wenn die Versorgungsspannung auf der 12V Seite abfllt,   Vektor Nr.3	      */
/* Es werden die aktuellen Frequenzen der momentanen Sendefrequenz und der      */
/* Frequenzen der beiden Baken abgespeichert, auerdem die akt. Einstellungen   */
/********************************************************************************/      
void ausschalten(void)
  { for (i=0; i<12; i++)
  	EEPROMwrite (i, uTxRx.Byte[i]);
    EEPROMwrite (12, bAkt_Einstellung);       	    	  	
  }   
/********************************************************************************/
/* Interruptroutine wird aller 50 ms vom Timer aufgerufen, Vektor Nr.5          */
/********************************************************************************/  
 void timer(void)
 {
   // In jedem 50 ms - Intervall:
   // Sichern der momentanen Tasterstellung	
     bGelesen = ~PINC; // negiert weil 0, wenn Kontakt geschlossen
     
     if (!(bTaster_v^(bGelesen))) // wenn sich mindestens ein Bit geaendert hat
     	{bTaster ^=((bGelesen)^bTaster_k)&(bGelesen);
     	                          // Die Bits tongeln, wenn 0/1 Flanke endeckt wird
     	 bTaster_k = bGelesen;	  // frs naechste Mal sicherstellen
     	 bTaster &= 0x7F;   //Bit 7 zu Null machen, Tx Spannung
     	 bTaster |=((~bGelesen)& 0x80);
     	}	 				
     bTaster_v = bGelesen ;	// gesichert frs Entprellen
     // In bTaster stehen jetzt die gueltigen Zustaende der Taster.
     // 1 bedeutet Taster ist gedrueckt

     bTicks++;		// Hochzhlen des Durchlaufzaehlers 0..1..2..3->0
     if (bTicks<4)
     	return;		// bei 50, 100, 150ms raus 
     			// bei 200 ms gehts weiter...
     			
     			
//*** In jedem 200ms - Intervall:********************************************** 
     bTicks=0;
//Bearbeiten eines evt. Tastendrucks auf SP1 setzen/lesen, SP2 stzen/lesen 
     if (bTaster & SP1_set)	//..Wenn SP1 setzen gedrueckt wurde
     // Speichern der aktuellen Displayfrequenz (Sende-und Empfangsfrequenz)
     // im EEPROM
     { for (i=0; i<4; i++)
  		      EEPROMwrite (i+13, uTxRx.Byte[i]);
       bTaster &= ~SP1_set;  // Bit fuer gedrueckten Schalter zuruecksetzen
     }		
     if (bTaster & SP2_set)	//..Wenn SP2 setzen gedrueckt wurde
     {
     	for (i=0; i<4; i++)
  		     EEPROMwrite (i+17, uTxRx.Byte[i]);
	    bTaster &= ~SP2_set;  // Bit fuer gedrueckten Schalter zuruecksetzen
     }		    
     if (bTaster & SP1_read)	//..Wenn SP1 lesen gedrueckt wurde
     {       	
     	for (i=0; i<4; i++)
  		     uTxRx.Byte[i] = EEPROMread(i+13);
      bTaster &= ~SP1_read;  // Bit fuer gedrueckten Schalter zuruecksetzen
      
      bAkt_Einstellung &= 0xE7; // kein RIT, keine Baken
      bParam = 0xC0;    // zum AD schicken, Freq:1.Zeile
      bNormal_OK = 0;	   // Neuaufbau des Displays veranlassen

     }		          
     if (bTaster & SP2_read)	//..Wenn SP2 lesen gedrueckt wurde
     {
     	for (i=0; i<4; i++)
  		    uTxRx.Byte[i] = EEPROMread(i+17);     	
	    bTaster &= ~SP2_read;
      bAkt_Einstellung &= 0xE7; // kein RIT,keine Baken 
      bParam = 0xC0;    //2m-Frequenz, zum AD schicken, Freq:1.Zeile	    
	    bNormal_OK = 0;	   // Neuaufbau des Displays veranlassen
     }
     	
//Auswerten des Tasters RIT

     if (bTaster & RIT)      // wenn RIT-Taste gedrueckt wurde
     {
     	bAkt_Einstellung ^= 0x08; 		   // Bit 3 tongeln: RIT-Bit
     	if (bAkt_Einstellung & 0x08)     //Wenn RIT eingeschaltet wurde..
     	     bAkt_Einstellung &= 0xEF;   //Baken raus
     	bNormal_OK = 0;		 // Neuaufbau erzwingen	 
     	bTaster &= ~RIT;
     }		
//Auswerten des Tasters Modus

     if (bTaster & Modus)      // wenn Modus-Taste gedrueckt wurde
     {
     	bModus = bAkt_Einstellung & 0x07;
     	bModus++;
     	if (bModus>2)
     	   bModus=0; 
     	bAkt_Einstellung &= 0xE0; //kein RIT-Betrieb und Baken-Betrieb mehr, 
     	                          //                   wenn Modus gewechselt
     	bAkt_Einstellung |= bModus; // Modus-Bits wieder einfuegen
     	
     	if (bModus > 0)  // Mikrowellenmode und 10GHz-Mode
     	    bBetriebsart_zeigen = 0;
     	else // 2m-Mode
     	{
     	    bBetriebsart_zeigen = 1;   
     	    bOK_Byte = 0;//Damit bei Wechsel in Mod 0 die Betriebsart neu 
      }             //                                aufgebaut wird
     	bNormal_OK = 0;		 // Neuaufbau erzwingen	   
     	bTaster &= ~Modus;
     }
//Auswerten des Tasters Baken

     if (bTaster & Baken)      // wenn Baken-Taste gedrueckt wurde
     {
     	bAkt_Einstellung ^= 0x10; 		// Bit 4 tongeln: Baken-Bit
     	if (bAkt_Einstellung & 0x10) // wenn Baken eingeschaltet wurde..
     	{
     	     bAkt_Einstellung &= 0xF7; // RIT-Anzeige ausschalten
           bBetriebsart_zeigen = 0;  // Betriebsartenanzeige aus   	     
     	}
     	//else bBetriebsart_zeigen = 1;   
     	bOK_Byte = 0;//Damit bei Wechsel die Betriebsart neu aufgebaut wird     
     	bNormal_OK = 0;		 // Neuaufbau erzwingen	 
     	bTaster &= ~Baken;
     }		     		     
// Beim ersten Einschalten und beim Umschalten der Bereiche ist die LCD neu 
//aufzubauen:  
//    	     
     if (!(bNormal_OK & Normal_OK))	//..wenn noch nicht auf LCD
     {
      clrscr();	
     	if (bAkt_Einstellung & 0x08)// Wenn RIT-Betrieb
     	{	 // Aufbau der 1.Zeile des Displays    	   
     	   bParam = 0xC0;	//2m, in 1.Zeile der LCD, zum AD schicken
     	   uRIT_Rel.lZahl = uTxRx.lZahl[0];     
     	   Drehgeberauswertung((unsigned long*)pRIT_Rel);
     	   // Aufbau der 2. Zeile des Displays
     	   bParam = 0xE0;	//2m, in 2.Zeile der LCD, zum AD schicken
     	   Drehgeberauswertung((unsigned long*)p2m_Rx);      
     	   if (bModus<2)  //im 2m- und im GHz-Mode
     	   {  	
     	        gotoxy(11,1);
     	        outtext("kHz");	
     	   }
	    }
	    else //Kein RIT-Betrieb
	    {
	    	     bParam = 0xC0; //2m, zum DDS Schicken, darstellen in 1. Zeile
	    	     Drehgeberauswertung((unsigned long*)p2m_Rx);
	    }	
	    if (bModus<2)  //im 2m- und im GHz-Mode
     	{  	
     	       gotoxy(11,0);
     	       outtext("kHz");	
     	}
      else //im 10 GHz Mode 
      {
      	gotoxy(1,0);
      	outtext("10.");
      }
      
      gotoxy(0,1);
      putchar2(0x30+bModus);// Ausgabe der Modnummer
      if (bAkt_Einstellung & 0x10) //wenn Baken gehoert 
      {
      	 gotoxy(10,1);
         outtext("Baken");	
      }                             
      
      bNormal_OK |= Normal_OK; // Bestaetigen 	
      }// Ende 1.LCD-Aufbau bei Normalbetrieb (RIT, Rel)

//Darstellen des "<-"-Symbols im RIT/Rel-Betrieb und setzen von bParam wenn 
//                                                          nicht RIT/Rel     
     if (bAkt_Einstellung & 0x08)// Wenn RIT oder Rel-Betrieb
     {
     	if (!(bTaster & Tx))//Rx zur Zeit
     	{
     		gotoxy (15,1);// 2.Zeile
        putchar2(0x7F);//bei RIT-Betrieb: "<-" 
     		gotoxy (15,0);// 1.Zeile
     		putchar2(0x20);// " "  
      }
	    else // Tx zur Zeit
	    {
     		gotoxy (15,0);// 1.Zeile
     		putchar2(0x7F);// "<-"
     		gotoxy (15,1);// 2.Zeile
     		putchar2(0x20);// " "    		
	    }	
     }  

     
//Tabelle der zul. Betriebsarten abarbeiten, wenn nicht am Rad gedreht wird 
     if ((!iSegmente) && (bBetriebsart_zeigen))		// Wenn nicht gedreht wurde und
     {  				
     	if (!(bAkt_Einstellung & 0x08)) //nicht RIT: Tabelle abarbeiten	
        {
          gotoxy(3,1);	
          if ((uTxRx.lZahl[0] >= 14400000) && (uTxRx.lZahl[0] < 14415000) 
          							&& (bOK_Byte!=1))
          {
          	outtext("  EME   ");
          	bOK_Byte = 1;
          }		
          if ((uTxRx.lZahl[0] >= 14415000) && (uTxRx.lZahl[0] < 14440000) 
          							&& (bOK_Byte!=2))
          {
          	outtext(" SSB, CW");
          	bOK_Byte = 2;
          }
          if ((uTxRx.lZahl[0] >= 14440000) && (uTxRx.lZahl[0] < 14450000) 
          							&& (bOK_Byte!=3))
          {          
          	outtext(" Baken  ");
          	bOK_Byte = 3;
          }	
          if ((uTxRx.lZahl[0] >= 14450000) && (uTxRx.lZahl[0] < 14480000)
          							&& (bOK_Byte!=4))
          {
          	outtext("  Alle  ");
          	bOK_Byte = 4;
          }
          if ((uTxRx.lZahl[0] >= 14480000) && (uTxRx.lZahl[0] < 14499000) 
          							&& (bOK_Byte!=5))
          {
          	outtext("Digital ");
          	bOK_Byte = 5;
          }		          		          	
          if ((uTxRx.lZahl[0] >= 14500000) && (uTxRx.lZahl[0] < 14520000) 
          							&& (bOK_Byte!=6))
          {
          	outtext("Rel Eing");
          	bOK_Byte = 6;
          }
          if ((uTxRx.lZahl[0] >= 14520000) && (uTxRx.lZahl[0] < 14560000) 
          							&& (bOK_Byte!=7))
          {
          	outtext("Simplex ");
          	bOK_Byte = 7;
          }
          if ((uTxRx.lZahl[0] >= 14560000) && (uTxRx.lZahl[0] < 14580000) 
          							&& (bOK_Byte!=8))
          {
          	outtext("Rel Ausg");
          	bOK_Byte = 8;
          }		     
          if ((uTxRx.lZahl[0] >= 14580000) && (uTxRx.lZahl[0] <= 14600000) 
          							&& (bOK_Byte!=9))
          {
          	outtext("Satelit ");
          	bOK_Byte = 9;
          }
          if ((uTxRx.lZahl[0] > 14600000) && (bOK_Byte!=10))
          {
          	outtext("        ");
          	bOK_Byte = 10;
          }		               		               		          				          		          
        }	
  	 //return;					// raus
     }//Ende Tabelle der zul. Betriebsarten abarbeiten, wenn nicht am Rad gedreht wird 	
     
  	if (bAkt_Einstellung & 0x08)// Wenn RIT-oder Rel-Betrieb
     	{
     		if (!(bTaster & Tx)) //bei Rx
     		{
     		   bParam = 0xE0;	//   schicken, 2.Zeile
     		   Drehgeberauswertung((unsigned long*)p2m_Rx); 	     			
     		}
     		else // bei Tx
     		{
     		   bParam = 0xC0;	//    schicken, 1.Zeile
     		   Drehgeberauswertung((unsigned long*)pRIT_Rel);	     			
     		}	
     		return;	
      }				
    if (bAkt_Einstellung & 0x10)  // Baken werden gehrt
    {  
    	 if(bModus > 0)//Mikrowellenmodi
       {  
       	 bParam = 0x40; // zum AD schicken, darstellen in 1. Zeile
       	 Drehgeberauswertung((unsigned long*)pBakenGHz); 
       }
       else  //2m-Modus
       {
       	 bParam = 0x40; // zum AD schicken, darstellen in 1. Zeile
       	 Drehgeberauswertung((unsigned long*)pBaken2m); 
       }
       return;
    }	
    else // keine Baken, kein RIT-Betrieb 
    {
    	 bParam = 0xC0;		 // schicken, 1.Zeile
       Drehgeberauswertung((unsigned long*)p2m_Rx);  
    }  
	  return;
  } 

//*****************************************************************************

  void main(void)
  {
 // Den verwendeten Pointern eine Adresse zuweisen 	
 p2m_Rx   = &uTxRx.lZahl[0];
 pBaken2m = &uTxRx.lZahl[1];
 pBakenGHz= &uTxRx.lZahl[2];
 pRIT_Rel = &uRIT_Rel;

//Ext. Interrupts fuer Port D, Bit 2 bei fallender Flanke einrichten,    Int 0  
//			                       Bit 3 bei aufsteigender Flanke einrichten,Int 1
  MCUCR|=0x0E;		
  				
  GIMSK|=0xC0;		//Int0 und Int1 aktivieren, noch sind gen. Int's gesperrt
  
//Richtungsregister programmieren  
  DDRA = 0xFF;		// Port A steuert das LCD-Display an, alles Ausgaenge 
  DDRB = 0xBF;		// Port B Bit 0 unbenutzt, Ausgang
  			          //        Bit 1 Master-Reset fr AD9951, Ausgang
  			          //        Bit 2 I/O UD (UpDate der Frequenz im AD9951), Ausg. 
  			          //        Bit 3 CSB ChipSelect fr ser.SPI-Bus, Ausg.
  			          //	      Bit 4 unbenutzt, Ausgang 
  			          //        Bit 5 Ausgang der SPI-Schnittst.(MOSI)
  			          //        Bit 6 Eingang der SPI-Schnittst. (MISO)
  			          //        Bit 7 Ausgang der SPI-Schnittst. (SCK) 	 
  DDRC = 0x00;		// Port C Bit 0 Eingang Set1-Taster
  			          //        Bit 1 Eingang Read1-Taster
  			          //        Bit 2 Eingang Set2-Taster
  			          //        Bit 3 Eingang Read2-Taster
  			          //        Bit 4 Eingang Read Bakenbereich-Taster
  			          //        Bit 5 Eingang Read Arbeitsbereich-Taster
  			          //        Bit 6 Eingang RIT-Taster
  			          //        Bit 7 Eingang Tx
  DDRD = 0x73;		// Port D Bit 0 unbenutzt, Ausgang
  			          //        Bit 1 unbenutzt, Ausgang
  			          //        Bit 2 Eingang Dehimpulsgeber,          Interrupt 0
  			          //        Bit 3 Eingang fuer Ausschalterkennung, Interrupt 1
  			          //        Bit 4 Ausgang WR\ LCD
  			          //        Bit 5 Ausgang RS  LCD
  			          //        Bit 6 Ausgang E   LCD
  			          //        Bit 7 Eingang Drehimpulsgeber Richtung
// Pull-Up-Widerstaende einschalten
 PORTC = 0x7F;    // Port C Bit 0 Pull Up Set1-Taster
  			          //        Bit 1 Pull Up Read1-Taster
  			          //        Bit 2 Pull Up Set2-Taster
  			          //        Bit 3 Pull Up Read2-Taster
  			          //        Bit 4 Pull Up Read Bakenbereich-Taster
  			          //        Bit 5 Pull Up Read Arbeitsbereich-Taster
  			          //        Bit 6 Pull Up RIT-Taster
  			          //        Bit 7 kein Pull Up am Eingang Tx  			
// SPI-Schnittstelle einrichten
  SPCR =0x52; 		// Bit 0 (SPR0) = 0 --> 8MHz/64 = 8us Takt, 125 kHz
  			          //     1 (SPR1) = 1 -->	8MHz/64 = 8us Takt, 125 kHz
  			          //                      0,32ms fuer 40 Byte FTW
  			          //     2 (CPHA) = 0 -->   
  			          //     3 (CPOL) = 0 --> Takt tief in Ruhe
  			          //     4 (MSTR) = 1 --> Master
  			          //     5 (DORD) = 0 --> MSB zuerst
  			          //     6 (SPE)  = 1 --> SPI ist enabled
  			          //     7 (SPIE) = 0 --> Kein Intrp., wenn Byte bertragen		

// Timer/Counter1 Betriebsweise einrichten und Interrupt erlauben  
 //Timer/Counter1 Kontrollregister B:  
  TCCR1B = 0x00;          // Stop Timer1
 //Timer/Counter1  (enthaelt den eigentlichen Timerinhalt, nach Vorteilung)        
  TCNT1H = 0x00;          // Ruecksetzen high-Teil
  TCNT1L = 0x00;	  //              low-Teil

  OCR1AH = 0x01;          // Set Compare A to 390 (186 hex)
  OCR1AL = 0x86;          // ((8MHz/1024)/390) = 50 ms Timer

  TIMSK = 0x40;           // Compare A Interrupt enable
 //Timer/Counter1 Kontrollregister B:
  TCCR1B = 0x0D;          // Start Timer1 mit clk/1024

  
// Im EEPROM gespeicherter Wert vom letzten Ausschalten zurcklesen

  for (i=0; i<12; i++)
  		uTxRx.Byte[i] = EEPROMread(i);         	
  bAkt_Einstellung = EEPROMread(12);
  
//Gelesene Werte auf Bereichsverletzung ueberpruefen
  if ((uTxRx.lZahl[1] < 14440000) || (uTxRx.lZahl[1] > 14450000)) 
         uTxRx.lZahl[1] = 14440000;
  if ((uTxRx.lZahl[2] < 14480000) || (uTxRx.lZahl[2] > 14500000)) 
         uTxRx.lZahl[2] = 14480000;         
  bAkt_Einstellung &= 0xE7; //Nach dem Einschalten RIT-Betrieb und Baken
                            //                               ausschalten
  bModus = bAkt_Einstellung & 0x07;
  if (bModus > 2) bModus = 0;//wenn EEPROM speichern fehlgeschlagen
	if (bModus > 0)  // Mikrowellenmode und 10GHz-Mode
     	     bBetriebsart_zeigen = 0;
  else bBetriebsart_zeigen = 1; //2m-Normal Mode   
     				               
// Initialisierung AD9951  
  PORTB = 0x0A;		// RESET = 1
// Da die SPI-bertragung mit MSB first ausgewaehlt wurde,
// muessen auch die hoeherwertigen Bytes zuerst bereitgestellt
// werden: 
  Steuerregister [0] = 0x00;	// schreiben, Adresse 0,Steuerregister Nr.1
  Steuerregister [1] = 0x00;  // Bits 31..24 
  Steuerregister [2] = 0x00;  // Bits 23..16 
  Steuerregister [3] = 0x06;	// Bits 15..8 
  Steuerregister [4] = 0x02;	// Bits 7..0 
  
  Frequenz1 [0] = 0x04;	// Adresse 4 -FTW, schreiben
  

  PORTB = 0x08;// Kein RESET mehr, CS hoch (inaktiv)

                     
  
  init_disp();			// Initialisierung des LCD-Schaltkreises
  clrscr();	  	 // LCD-Display loeschen  
          
  timer();		// nur einmal zu Beginn ist timer() aufzurufen,
 			        //    anschliessend geht es ueber Interrupt  
  SREG = 0x80; 		// SEI (Interruptfreigabe fuer alle drei Interr.)
  while(1)
    {
       ;
    }
  }     